package com.example.what_the_fantasy

import androidx.compose.runtime.Composable
import androidx.navigation.compose.rememberNavController
import com.example.what_the_fantasy.data.model.SrcLanguage
import org.junit.Assert.assertEquals
import org.junit.Test
import com.example.what_the_fantasy.data.services.IServices
import com.example.what_the_fantasy.data.services.ServicesStub
import com.example.what_the_fantasy.data.services.hashPassword
import org.junit.Assert.assertFalse
import org.junit.Assert.assertTrue

class UnitTestUser {
    private val services : IServices = ServicesStub()


    @Test
    fun testCreateUser_OK(){
        assertTrue(services.CreateUser("username", "email", "passwd")
        )
    }

    @Test
    fun testCreateUserSameUsername(){
        services.CreateUser("username2", "email2", "passwd")
        assertFalse(services.CreateUser("username2", "emai", "passwd"))
    }

    @Test
    fun testCreateUserSameEmail(){
        services.CreateUser("username3", "email3", "passwd")
        assertFalse(services.CreateUser("usernam", "email3", "passwd")
        )
    }

    @Test
    fun testUserEditUsername_OK(){
        services.CreateUser("username4", "email4", "passwd")
        assertTrue(services.EditUsername("newUsername",11))
    }

    @Test
    fun testUserEditUsernameSameUsernameWithAnotherUsername(){
        services.CreateUser("username5", "email5", "passwd")
        assertFalse(services.EditUsername("dev",10))
    }


    @Test
    fun testUserEditEmail_OK(){
        services.CreateUser("username6", "email6", "passwd")
        assertTrue(services.EditEmail("newEmail",11))
    }

    @Test
    fun testUserEditEmailSameUsernameWithAnotherEmail(){
        services.CreateUser("username7", "email7", "passwd")
        assertFalse(services.EditEmail("testeur@example.com",10))
    }

    @Test
    fun testUserEditPassword_OK(){
        services.CreateUser("username8", "email8", "passwd")
        services.EditPasswd("newPassword",11)
        assertEquals(hashPassword("newPassword"), services.getUserById(11)?.password)
    }

    @Test
    fun testUserChangeLangueVoAtVf(){
        services.CreateUser("username9", "email9", "passwd")
        services.ChangeLangage(11)
        assertEquals(SrcLanguage.vf, services.getUserById(11)?.langage)
    }

    @Test
    fun testUserChangeLangueVfAtVo(){
        services.CreateUser("username10", "email10", "passwd")
        services.ChangeLangage(11)
        services.ChangeLangage(11)
        assertEquals(SrcLanguage.vo, services.getUserById(11)?.langage)
    }

    @Test
    fun testUserExist_OK(){
        assertTrue(services.isUsernameExist("dev"))
    }

    @Test
    fun testUserNotExist_OK(){
        assertFalse(services.isUsernameExist("devNotExist"))
    }

    @Test
    fun testEmailExist_OK(){
        assertTrue(services.isEmailExist("testeur@example.com"))
    }

    @Test
    fun testEmailNotExist_OK(){
        assertFalse(services.isUsernameExist("email@example.com"))
    }

    @Test
    fun TestUserValidLogin_OK() {
        val navControllerMock: (Int) -> Unit = {}
        val initialiserCurrentUserMock: (Int) -> Unit = {}

        assertTrue(services.validLogin("dev", "1234", navControllerMock, initialiserCurrentUserMock))
    }

    @Test
    fun TestUserValidLoginBadUsername() {
        val navControllerMock: (Int) -> Unit = {}
        val initialiserCurrentUserMock: (Int) -> Unit = {}

        assertFalse(services.validLogin("devBad", "1234", navControllerMock, initialiserCurrentUserMock))
    }

    @Test
    fun TestUserValidLoginBadPassword() {
        val navControllerMock: (Int) -> Unit = {}
        val initialiserCurrentUserMock: (Int) -> Unit = {}

        assertFalse(services.validLogin("dev", "5678", navControllerMock, initialiserCurrentUserMock))
    }
}